#include "RenderState.h"
#include "Geometry.h"
#include "Material.h"


namespace CHCDemoEngine 
{

CGprofile RenderState::sCgFragmentProfile;
CGprofile RenderState::sCgVertexProfile; 


RenderState::RenderState():
mAlphaTestEnabled(false), 
mTexturesEnabled(false), 
mMode(RENDER),
mIsDepthPass(false)
{
	Reset(); 
}

	
bool RenderState::SetState(Mode mode)
{
	///////////
	//-- just change the modewhen necessary

	if (mode == mMode) return false;

	mMode = mode;

	if (mode == QUERY)
	{
		if (!mIsDepthPass)
		{
			glDisable(GL_LIGHTING);
			glColorMask(GL_FALSE, GL_FALSE, GL_FALSE, GL_FALSE);

			cgGLDisableProfile(sCgFragmentProfile);
			cgGLDisableProfile(sCgVertexProfile);
		}

		glDepthMask(GL_FALSE);

		SetState(false, false);
	}
	else // mode returns to render
	{
		/////////////
		//-- restore render state

		if (!mIsDepthPass)
		{
			glEnable(GL_LIGHTING);
			glColorMask(GL_TRUE, GL_TRUE, GL_TRUE, GL_TRUE);

			cgGLEnableProfile(sCgFragmentProfile);
			cgGLEnableProfile(sCgVertexProfile);
		}

		glDepthMask(GL_TRUE);
	}

	return true;
}

	
void RenderState::SetState(bool texturing, bool alphaTest)
{
	if (mAlphaTestEnabled && !alphaTest) 
	{
		mAlphaTestEnabled = false;
		glDisable(GL_ALPHA_TEST);
	}
	else if (!mAlphaTestEnabled && alphaTest)
	{
		mAlphaTestEnabled = true;
		glEnable(GL_ALPHA_TEST);
	}

	// allow only alpha textures in depth pass
	if (mTexturesEnabled && 
		(!texturing || mIsDepthPass && !mAlphaTestEnabled))
	{
		mTexturesEnabled = false;
		glDisable(GL_TEXTURE_2D);
		glDisableClientState(GL_TEXTURE_COORD_ARRAY);
	}
	else if (!mTexturesEnabled && texturing)
	{
		if (!mIsDepthPass || mAlphaTestEnabled)
		{
			mTexturesEnabled = true;
			glEnable(GL_TEXTURE_2D);
			glEnableClientState(GL_TEXTURE_COORD_ARRAY);
		}
	}
}


void RenderState::Reset()
{
	mCurrentVboId = -1;
	SetState(false, false);
	SetState(RENDER);
}


}